

def fwd_atm(spectra):
    """
    Given an instance of the spectra class function will forward model reflectances to top of atmosphere
    :param spectra: class instance of spectra
    :type spectra: object
    :return: updated spectra instance
    """
    return spectra